﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections.Generic;
using System.Collections.ObjectModel;

namespace PagedCollectionViewExample
{
  /// <summary>
  /// A paged collection view that uses a IPagedDataSource instance to asynchronously
  /// retrieve paged data from a server
  /// </summary>
  /// <typeparam name="T"></typeparam>
  public class ServerSidePagedCollectionView<T> : ObservableCollection<T>,
    IPagedCollectionView
  {
    #region fields

    private IPagedDataSource<T> _pagedDataSource;

    private int _pageSize;

    private bool _canChangedPage = true;

    private bool _isPageChanging;

    private int _itemCount;

    private int _totalItemCount;

    private int _pageIndex = 0;

    #endregion

    #region constructor

    public ServerSidePagedCollectionView(IPagedDataSource<T> pagedDataSource)
    {
      PageSize = 10;
      _pagedDataSource = pagedDataSource;
      RefreshData(0);
    }

    #endregion

    #region public properties

    public bool CanChangePage
    {
      private set
      {
        SetField(ref _canChangedPage, value, "CanChangePage");
      }
      get
      {
        return _canChangedPage;
      }
    }
       
    public bool IsPageChanging
    {
      private set
      {
        SetField(ref _isPageChanging, value, "IsPageChanging");
      }
      get
      {
        return _isPageChanging;
      }
    }

    public int ItemCount
    {
      private set
      {
        SetField(ref _itemCount, value, "ItemCount");
      }
      get
      {
        return _itemCount;
      }
    }

    public int TotalPages
    {
      get
      {
        return (int)Math.Ceiling((double)TotalItemCount / (double)PageSize);
      }
    }

    public int PageIndex
    {
      private set
      {
        SetField(ref _pageIndex, value, "PageIndex");
      }
      get
      {
        return _pageIndex;
      }
    }

    public int PageSize
    {
      set
      {
        SetField(ref _pageSize, value, "PageSize");
      }
      get
      {
        return _pageSize;
      }
    }
    
    public int TotalItemCount
    {
      private set
      {
        SetField(ref _totalItemCount, value, "TotalItemCount");
      }
      get
      {
        return _totalItemCount;
      }
    }

    #endregion

    #region public methods

    public bool MoveToFirstPage()
    {
      RefreshData(0);
      return true;
    }

    public bool MoveToLastPage()
    {
      RefreshData(TotalPages - 1);
      return true;
    }    

    public bool MoveToNextPage()
    {
      RefreshData(PageIndex + 1);
      return true;
    }

    public bool MoveToPage(int pageIndex)
    {
      RefreshData(pageIndex);
      return true;
    }

    public bool MoveToPreviousPage()
    {
      RefreshData(PageIndex - 1);
      return true;
    }

    #endregion

    #region events

    public event EventHandler<EventArgs> PageChanged;

    public event EventHandler<PageChangingEventArgs> PageChanging;

    protected void OnPageChanged()
    {
      if (PageChanged != null)
      {
        PageChanged(this, EventArgs.Empty);
      }
    }

    protected void OnPageChanging(int newPageIndex)
    {
      if (PageChanging != null)
      {
        PageChanging(this, new PageChangingEventArgs(newPageIndex));
      }
    }

    #endregion

    #region private implementation

    /// <summary>
    /// Fetches the data for the given page
    /// </summary>
    private void RefreshData(int newPageIndex)
    {
      // set the pre-fetch state
      CanChangePage = false;
      OnPageChanging(newPageIndex);

      _pagedDataSource.FetchData(newPageIndex, response =>
        {
          // process the received data
          DataReceived(response);

          // set the post-fetch state
          PageIndex = newPageIndex;
          OnPageChanged();
          CanChangePage = true;
        });
    }

    /// <summary>
    /// Updates the items exposed by this view with the given data
    /// </summary>
    private void DataReceived(PagedDataResponse<T> response)
    {
      TotalItemCount = ItemCount = response.TotalItemCount;
      this.ClearItems();

      foreach (var item in response.Items)
        this.Add(item);
    }

    /// <summary>
    /// Sets the given field, raising a PropertyChanged event
    /// </summary>
    private void SetField<T>(ref T field, T value, string propertyName)
    {
      if (object.Equals(field, value))
        return;

      field = value;
      OnPropertyChanged(new PropertyChangedEventArgs(propertyName));
    }

    #endregion
  }
}
